import numpy as np
import cv2
import os
import random
import pandas as pd


class ImageDataLoader():
    def __init__(self, data_path, data_name, imresize=False ,shuffle=False, pre_load=False):
        #pre_load: if true, all training and validation images are loaded into CPU RAM for faster processing.
        #This avoids frequent file reads. Use this only for small datasets.
        self.data_path = data_path
        self.data_name = data_name
        self.pre_load  = pre_load
        self.imresize  = imresize;

        self.data_files = [] ;
        self.label = [] ;
        filename = os.path.join(data_path,data_name)
        with open(filename, 'r') as file_to_read:
             idx = 0
             name = [] 
             while True:
                lines = file_to_read.readline()
                if not lines:
                  break
                  pass
                nameT, labelT=  lines.split()
                name.append(nameT) 
                self.label.append(labelT)
                if os.path.isfile(os.path.join(data_path,nameT)):
                   self.data_files.append( nameT)
        
        self.shuffle = shuffle
        
        if shuffle:
            random.seed(2468)
        self.num_samples = len(self.data_files)
        self.blob_list = {}        
        self.id_list = range(0,self.num_samples)
        if self.pre_load:
            print('Preloading the data This may take a while')
            idx = 0
            for fname in self.data_files:
                # first channel RGB image
                img = cv2.imread(os.path.join(self.data_path,fname),cv2.IMREAD_COLOR)
                img = img.astype(np.float32, copy=False)

                if self.imresize:
                    wd_1 = 227
                    ht_1 = 227
                # batchszie channel w h
                #RGB IMAGE
                img = cv2.resize(img,(wd_1,ht_1))
                img = img.transpose(2, 0, 1)
                img = img.reshape((1,img.shape[0],img.shape[1],img.shape[2]))

                # second channel dem  
                filename = os.path.join(self.data_path,fname)
                filename = filename.replace('R.jpg','D.txt')
                dem= []
                with open(filename, 'r') as file_to_read:
                    lines = file_to_read.readline()  
                    nameT, nC=  lines.split()
                    lines = file_to_read.readline()  
                    nameT, nR=  lines.split()

                    nR = int(nR)
                    nC = int(nC)
                    dem= np.zeros((nR,nC))     
                    for i in range(0,nR):
                       lines = file_to_read.readline()  
                       lines = lines.split()
                       for j in range(0,nC):
                           dem[i,j] = float(lines[j])
                if self.imresize:
                    wd_1 = 227
                    ht_1 = 227
                dem = cv2.resize(dem,(wd_1,ht_1))
                # flip dem, because dem is different dir with img
                dem= cv2.flip(dem,0)
                dem = dem.reshape((1,1,dem.shape[0],dem.shape[1]))
                
				# third channel slope  
                filename = os.path.join(self.data_path,fname)
                filename = filename.replace('R.jpg','S.txt')
                slope= []
                with open(filename, 'r') as file_to_read:
                    lines = file_to_read.readline()  
                    nC=  int(lines)  
                    lines = file_to_read.readline()  
                    nR=  int(lines)plot_confusion_matrix
                    slope= np.zeros((nR,nC))     

                    for i in range(0,nR):
                       lines = file_to_read.readline()  
                       lines = lines.split()

                       for j in range(0,nC):
                           slope[i,j] = float(lines[j])

                if self.imresize:
                    wd_1 = 227
                    ht_1 = 227
                slope = cv2.resize(slope,(wd_1,ht_1))
                slope= cv2.flip(slope,0)
                slope = slope.reshape((1,1,slope.shape[0],slope.shape[1]))

                # fifth channel grey texture 
                filename = os.path.join(self.data_path,fname)
                filename = filename.replace('R.jpg','RR.jpg')
                #print( filename ) 
                imgg = cv2.imread(filename,cv2.IMREAD_GRAYSCALE)
                imgg = imgg.astype(np.float32, copy=False)
                if self.imresize:
                    wd_1 = 227
                    ht_1 = 227
                # batchszie channel w h
                # GRAY IMAGE
                imgg = cv2.resize(imgg,(wd_1,ht_1))
                imgg = imgg.reshape((1,1,imgg.shape[0],imgg.shape[1]))
                
                blob = {}
                blob['data']      = img
                blob['imgg']      = imgg
                blob['dem']       = dem
                blob['slope']     = slope
                blob['label'] = int(self.label[idx]) 
                blob['fname'] = fname
                self.blob_list[idx] = blob
                idx = idx+1
                if idx % 100 == 0:                    
                     print( 'Loaded ', idx, '/', self.num_samples, 'files' )
               
            print('Completed Loading ', idx, 'files')

    

    def __iter__(self):
        if self.shuffle:            
           if self.pre_load:            
                random.shuffle(list(self.id_list))        
           else:
                random.shuffle(list(self.data_files))
        files = self.data_files
        id_list = self.id_list
       
        for idx in id_list:
            if self.pre_load:
                blob = self.blob_list[idx]    
                blob['idx'] = idx
            else:                    
                fname = files[idx]
                img = cv2.imread(os.path.join(self.data_path,fname),cv2.IMREAD_COLOR)

                ht = img.shape[0]
                wd = img.shape[1]
                ht_1 = (ht/4)*4
                wd_1 = (wd/4)*4

                if self.imresize:
                    wd_1 = 256
                    ht_1 = 256

                img = cv2.resize(img,(wd_1,ht_1))
                blob = {}
                blob['data']=img
                blob['label']=den
                blob['fname'] = fname
                print('fname ', fname )
            yield blob

    #def __len__(self):
    #    return len(self.files[self.split])
            
    def __len__(self):
        return self.num_samples



